'use strict';

require('core-js/modules/es6.typed.array-buffer');

require('core-js/modules/es6.typed.int8-array');

require('core-js/modules/es6.typed.uint8-array');

require('core-js/modules/es6.typed.uint8-clamped-array');

require('core-js/modules/es6.typed.int16-array');

require('core-js/modules/es6.typed.uint16-array');

require('core-js/modules/es6.typed.int32-array');

require('core-js/modules/es6.typed.uint32-array');

require('core-js/modules/es6.typed.float32-array');

require('core-js/modules/es6.typed.float64-array');

require('core-js/modules/es6.map');

require('core-js/modules/es6.set');

require('core-js/modules/es6.weak-map');

require('core-js/modules/es6.weak-set');

require('core-js/modules/es6.reflect.apply');

require('core-js/modules/es6.reflect.construct');

require('core-js/modules/es6.reflect.define-property');

require('core-js/modules/es6.reflect.delete-property');

require('core-js/modules/es6.reflect.get');

require('core-js/modules/es6.reflect.get-own-property-descriptor');

require('core-js/modules/es6.reflect.get-prototype-of');

require('core-js/modules/es6.reflect.has');

require('core-js/modules/es6.reflect.is-extensible');

require('core-js/modules/es6.reflect.own-keys');

require('core-js/modules/es6.reflect.prevent-extensions');

require('core-js/modules/es6.reflect.set');

require('core-js/modules/es6.reflect.set-prototype-of');

require('core-js/modules/es6.promise');

require('core-js/modules/es6.symbol');

require('core-js/modules/es6.object.freeze');

require('core-js/modules/es6.object.seal');

require('core-js/modules/es6.object.prevent-extensions');

require('core-js/modules/es6.object.is-frozen');

require('core-js/modules/es6.object.is-sealed');

require('core-js/modules/es6.object.is-extensible');

require('core-js/modules/es6.object.get-own-property-descriptor');

require('core-js/modules/es6.object.get-prototype-of');

require('core-js/modules/es6.object.keys');

require('core-js/modules/es6.object.get-own-property-names');

require('core-js/modules/es6.object.assign');

require('core-js/modules/es6.object.is');

require('core-js/modules/es6.object.set-prototype-of');

require('core-js/modules/es6.function.name');

require('core-js/modules/es6.string.raw');

require('core-js/modules/es6.string.from-code-point');

require('core-js/modules/es6.string.code-point-at');

require('core-js/modules/es6.string.repeat');

require('core-js/modules/es6.string.starts-with');

require('core-js/modules/es6.string.ends-with');

require('core-js/modules/es6.string.includes');

require('core-js/modules/es6.regexp.flags');

require('core-js/modules/es6.regexp.match');

require('core-js/modules/es6.regexp.replace');

require('core-js/modules/es6.regexp.split');

require('core-js/modules/es6.regexp.search');

require('core-js/modules/es6.array.from');

require('core-js/modules/es6.array.of');

require('core-js/modules/es6.array.copy-within');

require('core-js/modules/es6.array.find');

require('core-js/modules/es6.array.find-index');

require('core-js/modules/es6.array.fill');

require('core-js/modules/es6.array.iterator');

require('core-js/modules/es6.number.is-finite');

require('core-js/modules/es6.number.is-integer');

require('core-js/modules/es6.number.is-safe-integer');

require('core-js/modules/es6.number.is-nan');

require('core-js/modules/es6.number.epsilon');

require('core-js/modules/es6.number.min-safe-integer');

require('core-js/modules/es6.number.max-safe-integer');

require('core-js/modules/es6.math.acosh');

require('core-js/modules/es6.math.asinh');

require('core-js/modules/es6.math.atanh');

require('core-js/modules/es6.math.cbrt');

require('core-js/modules/es6.math.clz32');

require('core-js/modules/es6.math.cosh');

require('core-js/modules/es6.math.expm1');

require('core-js/modules/es6.math.fround');

require('core-js/modules/es6.math.hypot');

require('core-js/modules/es6.math.imul');

require('core-js/modules/es6.math.log1p');

require('core-js/modules/es6.math.log10');

require('core-js/modules/es6.math.log2');

require('core-js/modules/es6.math.sign');

require('core-js/modules/es6.math.sinh');

require('core-js/modules/es6.math.tanh');

require('core-js/modules/es6.math.trunc');

require('core-js/modules/es7.array.includes');

require('core-js/modules/es7.object.values');

require('core-js/modules/es7.object.entries');

require('core-js/modules/es7.object.get-own-property-descriptors');

require('core-js/modules/es7.string.pad-start');

require('core-js/modules/es7.string.pad-end');

require('core-js/modules/web.timers');

require('core-js/modules/web.immediate');

require('core-js/modules/web.dom.iterable');

require('regenerator-runtime/runtime');

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var validate = require('./validate');

/**
 * Validator object.
 * Delegation pattern.
 */
var proto = {};

/**
 * Validator initializer,
 * you must call it before
 * validate your data.
 *
 * Example:
 *     let validator = Object.create(proto);
 *     validator.init();
 *
 * @return {object} - Validator object
 * @api public
 */
proto.init = function init() {
  this.data = {};
  this.errors = {};
  this.path = [];
  return this;
};

/**
 * Recursive walk through
 * json schema and data.
 * Validates the node's *data*,
 * removes extra properties and
 * stores the path for error reporting.
 *
 * Example:
 *     validator.visit({root: schema}, {root: data});
 *
 * @param {object} parentNode - Schema node.
 * @param {object} data - Data node to validate.
 * @param {object} [opt] - Optional options.
 * @param {string} opt.nodeName - Internal Node's name.
 * @param {object|Array} [cleanedData] - Internal data node to save the valid data.
 * @param {object} cleanedData.root - Internal root Node.
 * @return {object} - Validator object
 * @api public
 */
proto.visit = function visit(parentNode, data, opt, cleanedData) {
  data = data || {};
  opt = opt || {};
  cleanedData = cleanedData || {};

  if (opt.nodeName) this.path.push(opt.nodeName);

  var _iteratorNormalCompletion = true;
  var _didIteratorError = false;
  var _iteratorError = undefined;

  try {
    for (var _iterator = Object.keys(parentNode)[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
      var name = _step.value;

      var node = parentNode[name];
      var n = validate(node, data, name, this.path, this.errors);

      if (!n.isValid()) continue;

      if (typeof data[name] === 'undefined') continue;

      if (node.type == 'object') {
        cleanedData[name] = {};
        this.visit(node.properties, data[name], { nodeName: name }, cleanedData[name]);
        continue;
      }

      if (node.type == 'array') {
        cleanedData[name] = [];

        // TODO: for (let [index, item] of data[name].entries()) wont work on Node.js v4
        var _iteratorNormalCompletion2 = true;
        var _didIteratorError2 = false;
        var _iteratorError2 = undefined;

        try {
          for (var _iterator2 = data[name].keys()[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
            var index = _step2.value;

            this.visit(_defineProperty({}, index, node.items), // Node schema
            _defineProperty({}, index, data[name][index]), // Node data
            { nodeName: name }, cleanedData[name]);
          }
        } catch (err) {
          _didIteratorError2 = true;
          _iteratorError2 = err;
        } finally {
          try {
            if (!_iteratorNormalCompletion2 && _iterator2.return) {
              _iterator2.return();
            }
          } finally {
            if (_didIteratorError2) {
              throw _iteratorError2;
            }
          }
        }

        continue;
      }

      if (Array.isArray(cleanedData)) cleanedData.push(data[name]);else cleanedData[name] = data[name];
    }
  } catch (err) {
    _didIteratorError = true;
    _iteratorError = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion && _iterator.return) {
        _iterator.return();
      }
    } finally {
      if (_didIteratorError) {
        throw _iteratorError;
      }
    }
  }

  if (opt.nodeName) this.path.pop();

  if (this.path.length === 0) this.data = cleanedData.root;

  return this;
};

/**
 * Creates the validator and runs it.
 *
 * @param {object} schema - JSON schema.
 * @param {object} data - Data to validate.
 * @return {object}
 * @api public
 */
function validatorFactory(schema, data) {
  var validator = Object.create(proto).init().visit({ root: schema }, { root: data });

  return {
    isValid: Object.keys(validator.errors).length === 0,
    errors: validator.errors,
    data: validator.data
  };
}

/**
 * Expose validator factory.
 */
module.exports = validatorFactory;

/**
 * Expose validator.
 */
module.exports.validator = proto;